<?php

namespace Classifieds\Advertisings\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Classifieds\Advertisings\Http\Requests\AdvertisingRequest;
use Classifieds\Advertisings\Interfaces\AdvertisingRepositoryInterface;
use Classifieds\Advertisings\Models\Advertising;
use Classifieds\Advertisings\Forms\Advertising as Form;

/**
 * APIController  class for advertising.
 */
class AdvertisingAPIController extends BaseController
{

    /**
     * Initialize advertising resource controller.
     *
     * @param type AdvertisingRepositoryInterface $advertising
     *
     * @return null
     */
    public function __construct(AdvertisingRepositoryInterface $advertising)
    {
        parent::__construct();
        $this->repository = $advertising;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Classifieds\Advertisings\Repositories\Criteria\AdvertisingResourceCriteria::class);
    }

    /**
     * Display a list of advertising.
     *
     * @return Response
     */
    public function index(AdvertisingRequest $request)
    {
        return $this->repository
            ->setPresenter(\Classifieds\Advertisings\Repositories\Presenter\AdvertisingPresenter::class)
            ->paginate();
    }

    /**
     * Display advertising.
     *
     * @param Request $request
     * @param Model   $advertising
     *
     * @return Response
     */
    public function show(AdvertisingRequest $request, Advertising $advertising)
    {
        return $advertising->setPresenter(\Classifieds\Advertisings\Repositories\Presenter\AdvertisingListPresenter::class);
        ;
    }

    /**
     * Create new advertising.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AdvertisingRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('advertisings::advertising.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('advertisings/advertising/' . $advertising->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('advertisings/advertising');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the advertising.
     *
     * @param Request $request
     * @param Model   $advertising
     *
     * @return Response
     */
    public function update(AdvertisingRequest $request, Advertising $advertising)
    {
        try {
            $data = $request->all();

            $advertising->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('advertisings::advertising.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('advertisings/advertising/' . $advertising->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('advertisings/advertising/' . $advertising->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the advertising.
     *
     * @param Model   $advertising
     *
     * @return Response
     */
    public function destroy(AdvertisingRequest $request, Advertising $advertising)
    {
        try {
            $advertising->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('advertisings::advertising.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('advertisings/advertising/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('advertisings/advertising/' . $advertising->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
